package com.ruoyi.asset.service.impl;

import java.util.Date;
import java.util.List;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.asset.domain.TbAsset;
import com.ruoyi.asset.domain.TbAssetStatusRecord;
import com.ruoyi.asset.domain.TbPlace;
import com.ruoyi.asset.mapper.TbAssetMapper;
import com.ruoyi.asset.mapper.TbAssetStatusRecordMapper;
import com.ruoyi.asset.mapper.TbPlaceMapper;
import com.ruoyi.asset.utils.UserUtils;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.asset.mapper.TbAssetBorrowRecordMapper;
import com.ruoyi.asset.domain.TbAssetBorrowRecord;
import com.ruoyi.asset.service.ITbAssetBorrowRecordService;

import javax.annotation.Resource;

/**
 * 设备借用记录Service业务层处理
 * 
 * @author 原动力
 * @date 2023-03-27
 */
@Service
public class TbAssetBorrowRecordServiceImpl extends ServiceImpl<TbAssetBorrowRecordMapper, TbAssetBorrowRecord> implements ITbAssetBorrowRecordService
{
    private static final Logger log = LoggerFactory.getLogger(TbAssetBorrowRecordServiceImpl.class);

    @Autowired
    private TbAssetBorrowRecordMapper tbAssetBorrowRecordMapper;

    @Resource
    private TbAssetMapper tbAssetMapper;

    @Resource
    private TbPlaceMapper tbPlaceMapper;

    @Resource
    private TbAssetStatusRecordMapper tbAssetStatusRecordMapper;

    /**
     * 查询设备借用记录
     * 
     * @param id 设备借用记录主键
     * @return 设备借用记录
     */
    @Override
    public TbAssetBorrowRecord selectTbAssetBorrowRecordById(Long id)
    {
        return tbAssetBorrowRecordMapper.selectTbAssetBorrowRecordById(id);
    }

    /**
     * 查询设备借用记录列表
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 设备借用记录
     */
    @Override
    public List<TbAssetBorrowRecord> selectTbAssetBorrowRecordList(TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        return tbAssetBorrowRecordMapper.selectTbAssetBorrowRecordList(tbAssetBorrowRecord);
    }

    /**
     * 新增设备借用记录
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 结果
     */
    @Override
    public int insertTbAssetBorrowRecord(TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        return tbAssetBorrowRecordMapper.insertTbAssetBorrowRecord(tbAssetBorrowRecord);
    }

    /**
     * 修改设备借用记录
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 结果
     */
    @Override
    public int updateTbAssetBorrowRecord(TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        return tbAssetBorrowRecordMapper.updateTbAssetBorrowRecord(tbAssetBorrowRecord);
    }

    /**
     * 批量删除设备借用记录
     * 
     * @param ids 需要删除的设备借用记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetBorrowRecordByIds(Long[] ids)
    {
        return tbAssetBorrowRecordMapper.deleteTbAssetBorrowRecordByIds(ids);
    }

    /**
     * 删除设备借用记录信息
     * 
     * @param id 设备借用记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetBorrowRecordById(Long id)
    {
        return tbAssetBorrowRecordMapper.deleteTbAssetBorrowRecordById(id);
    }

    /**
     * 新增设备借用记录
     *
     * @param tbAssetBorrowRecord 设备借用记录信息
     * @return 结果
     */
    @Override
    public AjaxResult saveBorrowRecord(TbAssetBorrowRecord tbAssetBorrowRecord) {
        String assetBarCode = tbAssetBorrowRecord.getAssetBarCode();
        TbAsset tbAsset = tbAssetMapper.selectTbAssetByBarCode(assetBarCode);
        if (tbAsset == null) {
            return AjaxResult.error("资产设备不存在");
        }
        TbAssetBorrowRecord borrowRecord = query().eq("asset_bar_code", assetBarCode).orderByDesc("borrow_date").last("limit 1").one();
        if (borrowRecord != null && borrowRecord.getIsreturn() > 0) {
            return AjaxResult.error("当前资产设备已借出");
        }
        if (tbAsset.getStatus() != 1) {
            if (tbAsset.getStatus() == 0) {
                return AjaxResult.error("当前资产设备已故障");
            }
            if (tbAsset.getStatus() == 2) {
                return AjaxResult.error("当前资产设备已报废");
            }
        }
        String borrowPlaceName = tbAssetBorrowRecord.getBorrowPlaceName();
        TbPlace tbPlace = tbPlaceMapper.selectTbPlaceByName(borrowPlaceName);
        if (tbPlace == null) {
            return AjaxResult.error("借用地点不存在");
        }

        Long userId = UserUtils.getLoginUserId();
        tbAssetBorrowRecord.setUserId(userId);
        tbAssetBorrowRecord.setIsreturn(0L);
        tbAssetBorrowRecord.setActualReturnDate(null);

        boolean result = save(tbAssetBorrowRecord);

        if (result) {
            TbAssetStatusRecord tbAssetStatusRecord = tbAssetStatusRecordMapper.selectTbAssetStatusRecordByAssetBarCode(assetBarCode);
            if (tbAssetStatusRecord == null) {
                TbAssetStatusRecord statusRecord = new TbAssetStatusRecord();
                statusRecord.setAssetBarCode(assetBarCode);
                statusRecord.setBorrowNumber(1L);
                int insert = tbAssetStatusRecordMapper.insert(statusRecord);
                if (insert < 1) {
                    throw new ServiceException("借用失败");
                }
            } else {
                Long borrowNumber = tbAssetStatusRecord.getBorrowNumber();
                tbAssetStatusRecord.setBorrowNumber(borrowNumber+1);
                int update = tbAssetStatusRecordMapper.updateById(tbAssetStatusRecord);
                if (update < 1) {
                    throw new ServiceException("借用失败");
                }
            }
        }

        return result ? AjaxResult.success("借用成功", tbAssetBorrowRecord) : AjaxResult.error("借用失败");
    }

    @Override
    public AjaxResult updateBorrowRecordToReturn(String barCode) {
        if (StrUtil.isBlank(barCode)) {
            return AjaxResult.error("资产条形码不能为空");
        }
        TbAssetBorrowRecord borrowRecord = query().eq("asset_bar_code", barCode).orderByDesc("borrow_date").last("limit 1").one();
        if (borrowRecord == null) {
            return AjaxResult.error("查无借用记录");
        }
        if (borrowRecord.getIsreturn() > 0) {
            return AjaxResult.error("资产设备已归还");
        }
        Long borrowUserId = borrowRecord.getUserId();
        Long returnUserId = UserUtils.getLoginUserId();
        if (!borrowUserId.equals(returnUserId)) {
            return AjaxResult.error("非借用人本人归还");
        }
        borrowRecord.setIsreturn(1L);
        borrowRecord.setActualReturnDate(new Date());
        boolean result = updateById(borrowRecord);
        return result ? AjaxResult.success("归还成功") : AjaxResult.error("归还失败");
    }
}
